;; Generated package description from eev.el  -*- no-byte-compile: t -*-
(define-package "eev" "20230102" "Support for e-scripts (eepitch blocks, elisp hyperlinks, etc)" '((emacs "24.4")) :commit "0df1fb2a0ae9fbffb5f507ab84810bf561710dd7" :authors '(("Eduardo Ochs" . "eduardoochs@gmail.com")) :maintainer '("Eduardo Ochs" . "eduardoochs@gmail.com") :keywords '("lisp" "e-scripts") :url "http://angg.twu.net/#eev")
